#
#  METRONOM.X  realisiert ein elektronisches Metronom mit einem 8051
#
#  Das Gert wird ber eine 12-Tasten Telefontastatur (3x4 Matrix) bedient
#  und zeigt das aktuelle Tempo in einer 3-Stelligen LED Anzeige an.
#
#  Die Taste '#' fhrt in das Men (erkenntlich durch die Anzeige 1.2.3.).
#  Dort knnen die 3 Betriebsarten gewhlt werden:
#
#  1  Normales Metronom.
#  2  Einzhler, Zhlt 2 Takte ein : 1, 2, 1,2,3,4
#  3  Tempomessung, mit Taste '*' wird das Tempo vorgegeben, im Display
#     erscheint das Tempo in bpm.
#
#  Die Zifferntasten gestatten die Eingabe des Tempos (47..199 bpm)in den
#  Betriebsarten 1 und 2. Die Taste '*' hat hier die Bedeutung Start / Stop.
#
#
#  Copyright (c) 1994 Markus Bauer
#


INCLUDE stddef.x                 # allgemeine Konstanten
INCLUDE vardef.x                 # automatische Variablenplazierung
INCLUDE r8051.x                  # 8051 SFR Definitionen
INCLUDE dfunc.x                  # erweiterte 16 Bit Arithmetik


#
#  Variablendefinitionen der LED Anzeige
#

FBYTE DisplayValue    # Im Display darzustellender BPM Wert
FBYTE DigitNr         # aktuell auszugebende Ziffer
IBYTE Digit2          # Ziffer 2 (MSB)     Reihenfolge der Definition ist
IBYTE Digit1          # Ziffer 1           wichtig, da auf Digit auch als
IBYTE Digit0          # Ziffer 0 (LSB)     Array zugegriffen wird
FBIT  DecPt2          # Dezimalpunkt zu Ziffer 2
FBIT  DecPt1          # Dezimalpunkt zu Ziffer 1
FBIT  DecPt0          # Dezimalpunkt zu Ziffer 0


#
#  Variablendefinitionen der Tastenmatrix
#

IWORD KeyBits         # Nimmt das Bitmuster der gedrckten Tasten auf
FBYTE KeyCode         # Code der zuletzt gedrckten Taste: 0..9, '#' oder '*'
FBIT  KeyPressed      # Eine Taste ist derzeit gedrckt
FBIT  KeyValid        # Eine Taste wurde dekodiert, Ergebnis in KeyCode
FBYTE KeyDelay        # Zhler zum Entprellen der Tastatur

CODETABLE KeyTable    # bersetzungstabelle Tastennummer -> Code

   '#' , 9 , 6 , 3 ,  # Spalte 0
    0  , 8 , 5 , 2 ,  # Spalte 1
   '*' , 7 , 4 , 1 ,  # Spalte 2

ENDTABLE

#
#  Variablendefinitionen der Ablaufsteuerung
#

FBIT  Running                 # Das Metronom luft
FBIT  CountIn                 # Einzhlmodus wurde gewhlt
FBIT  SetMode                 # Betriebsarteinstellung ist aktiv
FBIT  GetSpeed                # Tempoeingabe ist aktiv
FBIT  Measure                 # BPM Memodus ist aktiv

FBYTE BPM                     # eingestelltes Tempo
FBYTE PreCount                # zur Kontrolle des Einzhlens
FBYTE Count                   # Zhlt die Ticks fr einen Beat
FBYTE CountReload             # Derzeitiges Tempo in Ticks
FBYTE BeepCount               # Zhler fr die Dauer des Beeps
IWORD AvgSum                  # fr Mittelwertsbildung bei Measure

47    CONSTANT MinBPM         # kleinstes Tempo, entspricht 255 Ticks
7AH   CONSTANT PreMask        # Einzhlmaske fr 1, 2, 1234
ECH   CONSTANT 5ms_H          # Reload Werte fr Timer 0 zur Erzeugung des 5ms
78H   CONSTANT 5ms_L          # Taktes, gelten fr 12 MHz Clock
6     CONSTANT BeepDuration   # Dauer des Tonsignals in Ticks



: _Timer1_ISR ( -> )
#
# Interrupt Service Routine fr Timer 1 zur Erzeugung eines Tonsignals
# Die Routine verndert keine Register und mu daher auch keine retten
#


   B2H , B4H ,           # CPL T0    ; Wechsle den Pegel an Pin T0
   32H ,                 # RETI      ; Return from Interrupt

;



: SetTimer0 ( -> )
#
#  Stellt Timer 0 fr eine Dauer von 5ms ein und gibt ihn frei. Dieser Takt
#  Wird fr alle Zeitmessungen des Programms, die Tastaturabfrage und den
#  Displaymultiplex verwendet
#


   5ms_H TH0!              # Timer 0 laden ...
   5ms_L TL0!
   1 TR0!                  # ... starten
   0 TF0!                  # und berlauf rcksetzen

;


: SetTimer1 ( -> )
#
#  Stellt Timer 1 fr die Tonerzeugung ein und gibt ihn frei
#

   00H TH1!                # F = Clock / 12 / 256, ca. 3kHz = 1.5 kHz Ton
   1 TR1!                  # Timer starten
   0 TF1!                  # berlauf rcksetzen

;



: Beep ( -> )
#
#  Lst einen Ton aus
#

   BeepDuration BeepCount!          # Ton Dauer vorwhlen
   1 ET1!                           # Interrrupt Timer 1 Freigeben

;




: BpmToTicks ( bpm -> ticks )
#
#  Rechnet einen BPM Wert in Ticks um
#

   >R DROP
   12000L 0 R> D/
   SWAP DROP

;


ALIAS\ BpmToTicks \ TicksToBPM
#
#  Die Umrechnung in der Anderen Richtung geht genau gleich




: DeleteLeadingZeroes ( -> )
#
#  Unterdrckt fhrende Nullen im Display, indem der Wert 0 durch FH
#  ersetzt wird
#

   Digit2 @ DUP 0FH AND IF
      DROP
   ELSE
      0FH OR Digit2 !
      Digit1 @ DUP 0FH AND IF
         DROP
      ELSE
         0FH OR Digit1 !
      ENDIF
   ENDIF

;


: ProcessDisplay ( -> )
#
#  Bearbeitet die Ausgabe der Displaydaten im Multiplex. Mit jedem Aufruf
#  von ProcessDisplay wird eine Ziffer ausgegeben. Vor der Ausgabe der ersten
#  Ziffer werden die Variablen Digit0..2 mit den an Port P1 auszugebenden
#  Bitkombinationen belegt.
#

   DigitNr@ 2 > IF                  # Begrenze den Ziffernindex auf 2
      0 DigitNr!
   ENDIF

   DigitNr@ IFNOT                   # Berechne die Bitmuster fr die drei
      DisplayValue@                 # Anzeigestellen
      DUP 10 MOD                    # niederwertigste Ziffer
      DecPt0@ IF  00H               # Bitmaske bestimmen
      ELSE        80H  ENDIF
      OR Digit0 !                   # -> Digit0
      10 /                          # nchste Ziffer
      DUP 10 MOD
      DecPt1@ IF  10H               # Bitmaske bestimmen
      ELSE        90H  ENDIF
      OR Digit1 !                   # -> Digit1
      10 /                          # nchste Ziffer
      10 MOD
      DecPt2@ IF  20H               # Bitmaske bestimmen
      ELSE        A0H  ENDIF
      OR Digit2 !                   # -> Digit2
      DeleteLeadingZeroes           # fhrende Nullen ausblenden
   ENDIF

   Digit0 DigitNr@ + @ P1!          # Gebe die aktuelle Ziffer an P1 aus

   DigitNr@ 1 + DigitNr!            # Nchste Ziffer

;


: ReadKeyboard ( -> )
#
#  Liest den Zustand der Tastatur in die 16-Bit Variable KeyBits ein.
#  Jeder Taste entspricht ein Bit in dieser Variable, das 1 wird, wenn
#  die Taste gedrckt ist
#

   0BFH P1!                   # Setze Port zum Einlesen der ersten Spalte
   P1@ NOT 0FH AND            # Lese Spalte ein
   KeyBits 1 + !              # Speichere im LSB von KeyBits
   0CFH P1!                   # Setze Port zum Einlesen der zweiten Spalte
   P1@ NOT 0FH AND            # Lese Spalte ein
   16 * KeyBits 1 + @ OR      # Verodere mit dem LSB von KeyBits
   KeyBits 1 + !
   0DFH P1!                   # Setze Port zum Einlesen der dritten Spalte
   P1@ NOT 0FH AND            # Lese Spalte ein
   KeyBits !                  # Speichere im MSB von KeyBits

;


: TestKeyBits ( -> n )
#
#  berprft, ob in KeyBits mindestens ein Bit gesetzt ist. n ist 0 wenn
#  kein Bit gesetzt war.
#

   KeyBits @
   KeyBits 1 + @ OR

;



: ProcessKeyboard ( -> )
#
#  Bearbeitet das Abfragen der Matrixtastatur. Mit jedem Aufruf wird die
#  ganze Tastatur gescannt. Wurde eine Taste erkannt, wird diese decodiert
#  und in der Variablen KeyCode abgelegt. Das Flag KeyValid wird gesetzt.
#  Folgende Aufrufe von ProcessKeyboard bleiben ohne Aktion, bis alle
#  Tasten losgelassen wurden und die Entprellzeit verstrichen ist.
#


  KeyDelay@ IF                  # Tastenentprellung aktiv ?

     KeyDelay@ 1 -              # Dekrementiere Entprellzhler
     KeyDelay!

  ELSE                          # Frage Tastatur ab

     ReadKeyboard

     TestKeyBits IF             # Ist eine Taste gedrckt ?
        KeyPressed@ IFNOT       # War zuvor keine Taste gedrckt?
           0                    # dekodiere Taste, ab Nummer 0
           BEGIN
              KeyBits D@        # Schiebe das Bitmuster um eins nach Rechts
              DSHR
              KeyBits D!
           TestKeyBits WHILE    # Solange noch Bits gesetzt sind
              1 +               # Inkrementiere Tastennummer
           REPEAT
           KeyTable KeyCode!    # Speichere Tastencode
           TRUE KeyPressed!     # und setze die Flags
           TRUE KeyValid!
        ENDIF
     ELSE
        KeyPressed@ IF          # Setze den Entprellzhler beim
           30 KeyDelay!         # Loslassen der Taste
           FALSE KeyPressed!
        ENDIF
     ENDIF

  ENDIF

;



: ChangeToRunning ( -> )
#
#  Initialisiert den Zustand RUNNING
#

   TRUE Running!             # RUNNING Mode
   FALSE Measure!
   FALSE SetMode!
   FALSE GetSpeed!

   CountIn@ IF               # Setze PreCount
      PreMask PreCount!
   ELSE
      0 PreCount!
   ENDIF

   BPM@ BpmToTicks           # Setze Count und CountReload
   DUP CountReload!
   Count!

   1 DecPt2!                 # Setze LEDs fr ersten Beat
   0 DecPt1!
   0 DecPt0!

   Beep                      # Ton zum ersten Beat

   BPM@ DisplayValue!        # Zeige BPM Wert an

;



: ChangeToStopped ( -> )
#
#  Initialisiert den Zustand STOPPED
#

   FALSE Running!             # STOPPED
   FALSE Measure!
   FALSE SetMode!
   FALSE GetSpeed!

   0 DecPt2!                 # Alle LEDs AUS
   0 DecPt1!
   0 DecPt0!
   BPM@ DisplayValue!        # Zeige BPM Wert an

;



: ChangeToMeasure ( -> )
#
#  Initialisiert den Zustand MEASURE
#

   FALSE Running!            # MEASURE
   TRUE  Measure!
   FALSE SetMode!
   FALSE GetSpeed!

   0 DecPt2!                 # Alle LEDs AUS
   0 DecPt1!
   0 DecPt0!
   0 DisplayValue!           # Zeige 0 an
   4 PreCount!               # Es wird ber 4 Takte gemittelt
   0L AvgSum D!              # Nchsten Takt vorbereiten

;



: ChangeToGetSpeed ( -> )
#
#  Initialisiert den Zustand GETSPEED
#

   FALSE Running!            # GETSPEED
   FALSE Measure!
   FALSE SetMode!
   TRUE  GetSpeed!

   0 DecPt2!                 # Alle LEDs AUS
   0 DecPt1!
   0 DecPt0!
   BPM@ DisplayValue!        # Zeige BPM Wert an

;



: ChangeToSetMode ( -> )
#
#  Initialisiert den Modus SETMODE
#

   FALSE Running!             # SETMODE
   FALSE Measure!
   TRUE  SetMode!
   FALSE GetSpeed!

   1 DecPt0!                  # Alle LEDs AN
   1 DecPt1!
   1 DecPt2!
   123 DisplayValue!          # Anzeige 1.2.3.

;



: ProcessGetSpeed ( -> )
#
#  Wird aufgerufen, wenn durch eine Zifferneingabe das Setzen eines
#  BPM Wertes gefordert wurde

   KeyValid@ IF                      # Taste gedrckt ?
      KeyCode@ '*' = IF              # * - Taste ?
         DisplayValue@ MinBPM < IF   # Begrenze den BPM Wert auf den
            MinBPM BPM!              # Minimalwert und speichere ihn
            MinBPM DisplayValue!     # in BPM
         ELSE
            DisplayValue@ BPM!
         ENDIF
         ChangeToRunning             # Wechsle in Modus RUNNING
      ELSE
         KeyCode@ '#' = IF           # # - Taste ?
            ChangeToSetMode          # Wechsle in Modus SETMODE
         ELSE
            DisplayValue@ 19 > IF    # Schnellstes Tempo ist 199 bpm
               KeyCode@ DisplayValue!
            ELSE
               DisplayValue@ 10 * KeyCode@ +
               DisplayValue!
            ENDIF
         ENDIF
      ENDIF
      0 KeyValid!
   ENDIF

;


: ProcessSetMode ( -> )
#
#  Wird aufgerufen, wenn durch die # - Taste die Moduswahl gefordert
#  wurde

   KeyValid@ IF                      # Taste gedrckt ?
      KeyCode@ 1 = IF                # Taste 1
         FALSE CountIn!
         ChangeToStopped
      ENDIF
      KeyCode@ 2 = IF                # Taste 2
         TRUE CountIn!
         ChangeToStopped
      ENDIF
      KeyCode@ 3 = IF                # Taste 2
         ChangeToMeasure
      ENDIF
      0 KeyValid!
   ENDIF

;



: ProcessStopped ( -> )
#
#  Wird aufgerufen, wenn das Metronom im Zustand STOPPED ist
#

   KeyValid@ IF                      # Taste gedrckt ?
      KeyCode@ '#' = IF              # Taste #
         FALSE KeyValid!
         ChangeToSetMode
      ELSE
         KeyCode@ '*' = IF           # Taste *
            FALSE KeyValid!
            ChangeToRunning
         ELSE                        # Zifferntaste
            ChangeToGetSpeed
         ENDIF
      ENDIF
   ENDIF

;



: ProcessRunning ( -> )
#
#  Wird aufgerufen, wenn das Metronom im Zustand RUNNING ist
#


   Count@ IF
      Count@ 1 - Count!
   ELSE
      DecPt2@ NOT DecPt2!
      DecPt0@ NOT DecPt0!
      CountReload@ Count!
      PreCount@ IF
         PreCount@ 1 AND IF
            Beep
         ENDIF
         PreCount@ SHR
         DUP IFNOT
            ChangeToStopped
         ENDIF
         PreCount!
      ELSE
         Beep
      ENDIF
   ENDIF


   KeyValid@ IF                      # Taste gedrckt ?
      KeyCode@ '#' = IF              # Taste #
         FALSE KeyValid!
         ChangeToStopped
         ChangeToSetMode
      ELSE
         KeyCode@ '*' = IF           # Taste *
            FALSE KeyValid!
            ChangeToStopped
         ELSE                        # Zifferntaste
            ChangeToGetSpeed
         ENDIF
      ENDIF
   ENDIF

;




: ProcessMeasure ( -> )
#
#  Wird aufgerufen, wenn das Metronom im Zustand MEASURE ist
#

   AvgSum D@                         # Zyklus Zhlen
   1L D+
   AvgSum D!

   KeyValid@ IF                      # Taste gedrckt ?
      KeyCode@ '#' = IF              # Taste#
         ChangeToSetMode
      ENDIF
      KeyCode@ '*' = IF              # Taste *
         Beep
         PreCount@ 1 -               # Beat Zhlen
         PreCount!
         PreCount@ IFNOT
            AvgSum D@ 4L D/          # Mittelwert ber 4 Schlge
            TicksToBPM               # in BPM umrechnen
            DUP DisplayValue!        # im Display und in BPM
            BPM!                     # Speichern
            0L AvgSum D!             # Nchsten Takt vorbereiten
            4 PreCount!
         ENDIF
      ENDIF
      FALSE KeyValid!
   ENDIF

;



: ProcessBeep ( -> )
#
#  Zhlt die Dauer des Beeps und stellt den Ton ab, wenn die Zeit abgelaufen
#  ist.
#

   BeepCount@ IF                  # Beep ist aktiv, solange BeepCount<>0
      BeepCount@ 1 -              # Zhle Tick
      BeepCount!
      BeepCount@ IFNOT            # War es der letzte Tick ?
         FALSE ET1!               # dann sperre Timer 1 Interrupt
         FALSE T0!                # und setze T0 auf Low, wegen Stronverbr.
      ENDIF
   ENDIF

;



: Process ( -> )
#
#  Macht alles, wird alle 5 ms einmal aufgerufen
#

   ProcessBeep                    # Bearbeite die Dauer des Tonsignals
   ProcessKeyboard                # Frage die Tastenmatrix ab
   ProcessDisplay                 # Stelle eine Ziffer im Multiplex dar

   SetMode@ IF                    # Je nach dem aktuellen Zustand rufe
      ProcessSetMode              # die entsprechende Bearbeitungsroutine
   ELSE                           # auf.
      Running@ IF
         ProcessRunning
      ELSE
         Measure@ IF
            ProcessMeasure
         ELSE
            GetSpeed@ IF
               ProcessGetSpeed
            ELSE
               ProcessStopped
            ENDIF
         ENDIF
      ENDIF
   ENDIF

;


: _Main
#
#  Hauptfunktion, wird mit RESET aufgerufen
#

  0 KeyPressed!                # Initialisiere die Variablen der Tastatur-
  0 KeyValid!                  # dekodierung
  0 KeyCode!
  0 KeyDelay!

  120 BPM!                     # Default 120 BPM

  21H TMOD!                    # Timer Modus: T0 Modus 1 (16 Bit Timer)
                               #              T1 Modus 2 (8 Bit Auto Reload)

  SetTimer0                    # Initialisiere Timer 0
  SetTimer1                    # Initialisiere Timer 1

  TRUE EA!                     # Gebe Interrupts frei

  ChangeToSetMode              # Das Programm beginnt mei dem Men

  BEGIN                        # Hauptschleife
     Process                   # alle Aktionen eines Ticks
     BEGIN TF0@ UNTIL          # warte auf Timer 0 berlauf
     SetTimer0                 # Initialisiere Timer 0 fr nchsten Tick
  POWER_OFF UNTIL

;


#
#  Erzeugen des EPROM-Files
#

$IFERR BYE $ENDIF            # Breche ab bei Fehler

0020HL COMPILE

RESET       _Main
3 INTERRUPT _Timer1_ISR

WRITE       TEST.HEX         # Erzeugen der HEX - Datei

BYE                          # und tsch ...

