#
#  SERIAL2.X   Interrupt-gesteuerter Treiber fr die interne serielle
#              Schnittstelle, hier fr Controller mit 256 Bytes internem
#              Speicher
#
#  PUTCHAR ( ch -> )  nimmt ein Zeichen vom Stapel und schreibt es in den
#     Sende-Ringpuffer
#  GETCHAR ( -> ch )  entnimmt dem Empfngspuffer ein Zeichen und legt es
#     auf den Stapel
#  CHARS_PRESENT ( -> n ) legt die Anzahl der im Empfangspuffer verfgbaren
#     Zeichen auf den Stapel. (Als ALIAS realisiert)
#  sInitBuffers ( -> )  initialisiert die internen Variablen des Moduls und
#     gibt den zur Schnittstelle gehrenden Interrupt frei. Mu zu Beginn des
#     Programms einmal aufgerufen werden.
#  _sInterruptService ( -> ) Interruptroutine des Treibers. Mu durch ein
#     INTERRUPT Statement nach dem Compilieren installiert werden.
#
#  Copyright (C) 1994 Markus Bauer
#


#  Definition der Schnittstellenpuffer am oberen Ende des internen Speichers

20H CONSTANT          sBufferSize       # je 32 Bytes Puffer
E0H VARIABLE          sRxBuffer         # Empfangspuffer von E0..FF
C0H VARIABLE          sTxBuffer         # Sendepuffer von C0..DF

#  Steuervariablen des Schnittstellentreibers (lokal, werden am Ende mit HIDE
#  versteckt)

FBIT                  tx_empty          # Ein TX Interrupt fand statt und es
                                        # waren keine Daten im Puffer
FBYTE                 tx_rd             # Lesezeiger des Sendepuffers
FBYTE                 tx_wt             # Schreibzeiger des Sendepuffers
FBYTE                 tx_n              # Anzahl Bytes im Sendepuffer
FBYTE                 rx_rd             # Lesezeiger des Empfangspuffers
FBYTE                 rx_wt             # Schreibzeiger des Empfangspuffers
FBYTE                 rx_n              # Anzahl Bytes im Empfangspuffer


ALIAS\ rx_n@ \ CHARS_PRESENT            # legt die Anzahl Zeichen im Empfangs-
                                        # puffer auf den Stapel




: PUTCHAR ( ch -> )
#
#  Schreibt ein Zeichen in den Sendepuffer und startet die Ausgabe wenn ntig
#


   BEGIN                                # Warte, bis Platz im Sendepuffer ist
      tx_n@ sBufferSize <
   UNTIL

   FALSE EA!                            # Interrupts Sperren

   tx_n@ 1 +  tx_n!                     # Anzahl Zeichen im Puffer erhhen
   tx_wt@ !                             # Zeichen in den Puffer schreiben
   tx_wt@ 1 +  tx_wt!                   # Schreibzeiger erhhen

   tx_wt@                               # Ende des Puffers ?
   sTxBuffer sBufferSize +
   = IF
      sTxBuffer tx_wt!                  # Umbrechen
   ENDIF

   tx_empty@ IF                         # Ausgabe neu anstoen, falls diese
      FALSE tx_empty!                   # durch einen leeren Puffer unter-
      TRUE TI!                          # brochen war
   ENDIF

   TRUE EA!                             # Interrupts freigeben

;



: GETCHAR ( -> ch )
#
#  Liest ein Zeichen aus dem Empfangpuffer, wenn eines darin ist
#

   BEGIN                                # Warte, bis ein Zeichen im Empfangs
      rx_n@                             # Puffer ist
   UNTIL

   FALSE EA!                            # Interrupts Sperren

   rx_n@ 1 -  rx_n!                     # Anzahl Zeichen im Puffer dekrem.
   rx_rd@ @                             # Zeichen aus dem Puffer lesen
   rx_rd@ 1 +  rx_rd!                   # Lesezeiger erhhen

   rx_rd@                               # Ende des Puffers ?
   sRxBuffer sBufferSize +
   = IF
      sRxBuffer rx_rd!                  # Umbrechen
   ENDIF

   TRUE EA!                             # Interrupts freigeben

;



: sInitBuffers ( -> )
#
#  Initialisiert die Variablen dieses Moduls und gibt den Interrupt frei.
#  Die Serielle Schnittstelle mu getrennt initialisiert werden
#  (Baudrate, Datenformat etc.)
#

   TRUE         tx_empty!               # Initialisieren der Flags
   FALSE        RI!
   FALSE        TI!

   sTxBuffer    tx_rd!                  # Initialisieren der Puffer-
   sTxBuffer    tx_wt!                  # zeiger
   0            tx_n!
   sRxBuffer    rx_rd!
   sRxBuffer    rx_wt!
   0            rx_n!

   TRUE         ES!                     # Interrupt freigeben
   TRUE         EA!

;



: _sInterruptService
#
#  Interruptfunktion des Treibers
#

   Save_Registers

   TI@ IF                                # TX Interrupt ?
      tx_n@ IF                           # Zeichen im Puffer ?
         tx_n@ 1 -  tx_n!                # Anzahl Zeichen im Puffer dekrem.
         tx_rd@ @ SBUF!                  # Zeichen aus dem Puffer lesen
         tx_rd@ 1 +  tx_rd!              # Lesezeiger erhhen
         tx_rd@                          # Ende des Puffers ?
         sTxBuffer sBufferSize +
         = IF
            sTxBuffer tx_rd!             # Umbrechen
         ENDIF
      ELSE
         TRUE tx_empty!                  # Puffer war leer !
      ENDIF
      FALSE TI!                          # Int-Anforderung rcksetzen
   ENDIF

   RI@ IF                                # RX Interrupt
      rx_n@ sBufferSize < IF             # Platz im Puffer ?
         rx_n@ 1 +  rx_n!                # Anzahl Zeichen im Puffer erhhen
         SBUF@ rx_wt@ !                  # Zeichen in den Puffer schreiben
         rx_wt@ 1 +  rx_wt!              # Schreibzeiger erhhen
         rx_wt@                          # Ende des Puffers ?
         sRxBuffer sBufferSize +
         = IF
            sRxBuffer rx_wt!             # Umbrechen
         ENDIF
      ENDIF
      FALSE RI!                          # Int-Anforderung rcksetzen
   ENDIF

   Restore_Registers
   InterruptReturn

;



#
#  Lokale Symbole, fr die folgenden Programmteile unsichtbar
#

HIDE   tx_empty!       HIDE   tx_empty@
HIDE   tx_rd!          HIDE   tx_rd@
HIDE   tx_wt!          HIDE   tx_wt@
HIDE   tx_n!           HIDE   tx_n@
HIDE   rx_rd!          HIDE   rx_rd@
HIDE   rx_wt!          HIDE   rx_wt@
HIDE   rx_n!


